﻿/**
OS情報。

Windows専用だけど。
*/
module nemuxi.negui.environ.os;

debug import std.stdio: wl = writefln, pl = printf;
debug(os) void main() {}

import std.contracts;

import win32.windows;

import nemuxi.negui.system.base;
import nemuxi.negui.environ.environ;

///
class OSException: EnvironException {
	mixin MixInNeGuiException;
}

enum {
	SM_SERVERR2 = 89,
	VER_SUITE_WH_SERVER = 0x00008000,
}

class OS: IInitialize, IText {
	private {
		OSVERSIONINFOEX OSInfo;
	}

	override void initialize() {
		OSInfo.dwOSVersionInfoSize = OSInfo.sizeof;
		enforce(GetVersionEx(cast(OSVERSIONINFO*)&OSInfo), new OSException(Err.text));
	}

	mixin(ClassGetSet!(int)("majorVersion", q{OSInfo.dwMajorVersion}, false));
	mixin(ClassGetSet!(int)("minorVersion", q{OSInfo.dwMinorVersion}, false));
	mixin(ClassGetSet!(int)("buildNumber", q{OSInfo.dwBuildNumber}, false));
	mixin(ClassGetSet!(int)("platformId", q{OSInfo.dwPlatformId}, false));
	Text servicePack() {
		return Text(OSInfo.szCSDVersion.ptr);
	}
	mixin(ClassGetSet!(WORD)("servicePackMajor", q{OSInfo.wServicePackMajor}, false));
	mixin(ClassGetSet!(WORD)("servicePackMinor", q{OSInfo.wServicePackMinor}, false));
	enum SUITES: WORD {
		SMALLBUSINESS            = VER_SUITE_SMALLBUSINESS            , /// Microsoft Small Business Serverがインストールされています。または、かつてインストーすれさていましたが、別のバージョンのWindowsにアップグレードされている可能性があります。
		ENTERPRISE               = VER_SUITE_ENTERPRISE               , /// Windows NT 4.0 Enterprise Edition、Windows 2000 Advanced Server、Windows Server 2003 Enterprise Edition、Windows Server 2008 Enterpriseのいずれかがインストールされています。
		BACKOFFICE               = VER_SUITE_BACKOFFICE               , /// Microsoft BackOfficeコンポーネントがインストールされています。
		TERMINAL                 = VER_SUITE_TERMINAL                 , /// ターミナルサービスがインストールされています。VER_SUITE_TERMINALがセットされていて、かつ、VER_SUITE_SINGLEUSERTSがセットされていない場合には、システムはアプリケーションサーバモードで動作しています。
		SMALLBUSINESS_RESTRICTED = VER_SUITE_SMALLBUSINESS_RESTRICTED , /// Microsoft Small Business Serverが制限クライアントライセンスでインストールされています。
		EMBEDDEDNT               = VER_SUITE_EMBEDDEDNT               , /// Windows XP Embeddedがインストールされています。
		DATACENTER               = VER_SUITE_DATACENTER               , /// Windows 2000 Datacenter Server、Windows Server 2003 Datacenter Edition、Windows Server 2008 Datacenterのいずれかがインストールされています。
		SINGLEUSERTS             = VER_SUITE_SINGLEUSERTS             , /// リモートデスクトップがサポートされていますが、1つの対話型セッションのみがサポートされています。この値はシステムがアプリケーションサーバモードで動作していない場合にはセットされません。
		PERSONAL                 = VER_SUITE_PERSONAL                 , /// Windows XP Home Edition、Windows Vista Home Basic、Windows Vista Home Premiumのいずれかがインストールされています。
		BLADE                    = VER_SUITE_BLADE                    , /// Windows Server 2003 Web Editionがインストールされています。
		STORAGE_SERVER           = VER_SUITE_STORAGE_SERVER           , /// Windows Storage Server 2003またはWindows Storage Server 2003 R2がインストールされています。
		COMPUTE_SERVER           = VER_SUITE_COMPUTE_SERVER           , /// Windows Server 2003 Compute Cluster Editionがインストールされています。
		WH_SERVER                = VER_SUITE_WH_SERVER                , /// Windows Home Serverがインストールされています。
	}
	mixin(ClassGetSet!(SUITES)("suites", q{OSInfo.wSuiteMask}, false));

	//wProductType
	enum PRODUCTTYPE: BYTE {
		WORKSTATION       = VER_NT_WORKSTATION,       /// システムはWindows NT 4.0 Workstation、Windows 2000 Professional、Windows XP Home Edition、Windows XP Professional、Windows Vi
		DOMAIN_CONTROLLER = VER_NT_DOMAIN_CONTROLLER, /// システムはドメインコントローラです。
		SERVER            = VER_NT_SERVER,            /// システムはサーバです。ドメインコントローラでもあるサーバは、VER_NT_SERVERではなくVER_NT_DOMAIN_CONTROLLERとして報告されます。
	}
	mixin(ClassGetSet!(PRODUCTTYPE)("productType", q{OSInfo.wProductType}, false));

	/**
	何でもいいからだれかmoduleくれ。
	*/
	override const Text toText() {
		immutable Major=majorVersion;
		immutable Minor=minorVersion;
		
		const Version=Text("%s.%s", Major, Minor);

		assert(Major >= 5, "2000以上のみってことで。むしろ2000も弾きたい");
		string Name;
		switch(Major) {
			case 5: // 2k, XP, 2003, 2003 R2
				switch(Minor) {
					case 0: // 2000
						Name = "2000";
						break;
					case 1: // XP
						Name = "XP";
						break;
					case 2: // 2003, 2003 R2
						Name = "2003";
						if(GetSystemMetrics(SM_SERVERR2)) {
							Name ~= " R2";
						}
						break;
					default:
						Name = "?-1";
				}
				break;
			
			case 6: // Vista ,Server 2008, Server 2008 R2, 7
				switch(Minor) {
					case 0: // Vista, Server 2008
						if(productType == PRODUCTTYPE.WORKSTATION) {
							Name = "Vista";
						} else {
							Name = "Server 2008";
						}
						break;
					case 1: // Server 2008 R2, 7
						if(productType == PRODUCTTYPE.WORKSTATION) {
							Name = "7";
						} else {
							Name = "Server 2008 R2";
						}
						break;
					default:
						Name = "?-2";
				}
				break;
			
			default: // 未来
				Name = "?-3";
		}

		//サービスパック
		Text ServicePack;
		if(servicePackMajor) {
			ServicePack = Text("%s.%s", servicePackMajor, servicePackMinor);
		} else {
			ServicePack = "none";
		}

		return Text(
			InfoTextLine("Windows", Name),
			Text.newline,
			InfoTextLine("Version", Version),
			Text.newline,
			InfoTextLine("ServicePack", ServicePack),
			Text.newline,
			InfoTextLine("BuildNumber", buildNumber),
			Text.newline,
			Text("Suites: %08X", suites),
			Text.newline,
			InfoTextLine("ProductType", productType)
		);
	}

	override string toString() {
		return toText.text8;
	}
}
debug(os) unittest {
	auto o=new OS;
	o.initialize;
	wl(o.toText.toString);
}



